unit complex_sub;

interface

uses Velthuis.BigIntegers, Velthuis.Bigdecimals;

type
  combig = record                     // f\
    r : bigdecimal;
    i : bigdecimal;
  end;

var
  zero, one : bigdecimal;             // 0, 1
  precisions : integer;               // L
  paib : bigdecimal;                  // 
  maxmpfbig : Bigdecimal;             // (0)l ]:̑p
  delt : bigdecimal;                  // Ɍlp
  log_2pis2 : combig;                 // Ln(2)/2
  ctwo, cone : combig;                // 2+0i, 1+0i

const
  DPS = 200;                          // ZL
  NB = 120;                           // xk[C  z NB + 1
  EPSC = '1e-100';                    // log_big Βl@ln(x)
  EPS = '1e-60';                      // ^􉽊֐l
  DELTB = '1e-30';                    // Ɍl

function combiground(x : combig): combig;
function cadd(a, b: comBig): ComBig;
function csub(a, b: ComBig): ComBig;
function caddb(a: combig; b:bigdecimal): combig;
function csubb(a : combig; b: bigdecimal): combig;
function cmulb(a: Combig; b: bigdecimal): ComBig;
function cmul(a, b: Combig): ComBig;
function cdivb(a: Combig; b: bigdecimal): Combig;
function cdiv(a, b: Combig): Combig;
function caeqb(x, y : combig): boolean;
function cabs(a : combig): bigdecimal;
function gammabig(xx : combig; var ans: combig) : integer;
function pow_big(x, y : combig; eps : bigdecimal): combig;
function pi_big: Bigdecimal;
function log_big(z : combig; eps : bigdecimal; dpcs: integer): combig;

implementation

//------------------------------  comprex calc-------------------------------------------
var
  z10 : combig;
  Z10F : boolean = false;
  BM : array[0..NB] of Bigdecimal;    // xk[Cz
  tmpb: combig;
  epsb : bigdecimal;

// x: combig dpcsɊۂ
function combiground(x : combig): combig;
begin
  result.r := x.r.RoundToPrecision(precisions);
  result.i := x.i.RoundToPrecision(precisions);
end;

// ComBigf̉Z
function cadd(a, b: comBig): ComBig;
begin
  result.r := a.r + b.r;
  result.i := a.i + b.i;
end;

// ComBigf̌Z
function csub(a, b: ComBig): ComBig;
begin
  result.r := a.r - b.r;
  result.i := a.i - b.i;
end;

// combig + bigdecimal
function caddb(a: combig; b:bigdecimal): combig;
begin
  result.r := a.r + b;
  result.i := a.i;
end;

// combig - bigdecimal
function csubb(a : combig; b: bigdecimal): combig;
begin
  result.r := a.r - b;
  result.i := a.i;
end;

// Combig bigdecimal̏Z
function cmulb(a: Combig; b: bigdecimal): ComBig;
begin
  result.r := a.r * b;
  result.i := a.i * b;
end;

// Combigf̏Z
function cmul(a, b: Combig): ComBig;
begin
  result.r := a.r * b.r - a.i * b.i;
  result.i := a.r * b.i + a.i * b.r;
  result := combiground(result);
end;

// Combig bigdecimal̏Z
function cdivb(a: Combig; b: bigdecimal): Combig;
begin
  if b <> Bigdecimal.Zero then begin
    result.r := a.r / b;
    result.i := a.i / b;
  end
  else begin
    result.r := Bigdecimal.Zero;
    result.i := Bigdecimal.Zero;
  end;
end;

// Combigf̏Z
function cdiv(a, b: Combig): Combig;
var
  bb, arbraibi, aibrarbi: Bigdecimal;
begin
  bb := b.r * b.r + b.i * b.i;
  arbraibi := a.r * b.r + a.i * b.i;
  aibrarbi := a.i * b.r - a.r * b.i;
  bb := bb.RoundToPrecision(precisions);
  arbraibi := arbraibi.RoundToPrecision(precisions);
  aibrarbi := aibrarbi.RoundToPrecision(precisions);

  if bb <> Bigdecimal.Zero then begin
    result.r := arbraibi / bb;
    result.i := aibrarbi / bb;
  end
  else begin
    result.r := Bigdecimal.Zero;
    result.i := Bigdecimal.Zero;
  end;
end;

// a:combig = b:combig
function caeqb(x, y : combig): boolean;
begin
  if (x.r = y.r) and (x.i = y.i) then result := true
                                 else result := false;
end;

// Combig̐Βl
function cabs(a : combig): bigdecimal;
var
  arh2, aih2 : bigdecimal;
  tmp : bigdecimal;
begin
  arh2 := a.r * a.r;
  aih2 := a.i * a.i;
  arh2 := arh2.RoundToPrecision(precisions);
  aih2 := aih2.RoundToPrecision(precisions);
  tmp := arh2 + aih2;
  result := bigdecimal.Sqrt(tmp, precisions);
end;

//------------------------------------------------------------------------------
// ő  [Nbȟݏ@ BigInteger
function gcd_BigInteger(x, y: BigInteger): BigInteger;
var
  t : BigInteger;
begin
  while y <> 0 do begin
    t := x mod y;
    x := y;
    y := t;
  end;
  result := x;
end;

// 
// https://www.kurims.kyoto-u.ac.jp/~ooura/pi_fft-j.html
// T.Ooura AGM ASY
function pi_big: Bigdecimal;
var
  n, dpcs : integer;
  rmback : BigDecimal.RoundingMode;
  SQRT_SQRT_EPSILON, c, b, e : BigDecimal;
  npow : Bigdecimal;
  a, one, two, four, five, eight : Bigdecimal;
begin
  rmBack := BigDecimal.DefaultRoundingMode;           // Zۂ߃[hobNAbv
  dpcs := precisions + 5;                             // w萸x + 
  BigDecimal.DefaultPrecision := dpcs;
  BigDecimal.DefaultRoundingMode := rmNearesteven;

  one := '1';
  one := one.RoundToPrecision(dpcs);                  // one̗Ldpcsɐݒ
  two := '2';
  four := '4';
  five := '5';
  eight := '8';
  SQRT_SQRT_EPSILON := one / BigDecimal.IntPower(two, dpcs shl 0); // l
  c := BigDecimal.Sqrt(one / eight, dpcs * 2);
  a := one + c + c + c;
  b := BigDecimal.Sqrt(a, dpcs * 2);
  e := b - five / eight;
  b := b + b;
  c := e - c;
  a := a + e;
  npow := '4';
  n := 0;
  while (e > SQRT_SQRT_EPSILON) and (n < 100) do begin
    npow := npow + npow;                              // 8,16,32,64
    e := (a + b) / two;                               // ϒl ZLł̊ۂ
    b := a * b;
    b := b.RoundToPrecision(dpcs);                    // pcs +  ۂ
    b := BigDecimal.Sqrt(b, dpcs * 2);                // Lł̊ۂ
    e := e - b;
    b := b + b;
    c := c - e;
    a := e + b;
    inc(n);
  end;
  e := e * e;
  e := e.RoundToPrecision(dpcs);                      // pcs +  ۂ
  e := e / four;                                      // e = e * e / 4
  a := a + b;
  result := (a * a - e - e / two) / (a * c - e) / npow;   // Z̏Ԃɒ
  result := result.RoundToPrecision(precisions);             // w̐xɊۂ

  BigDecimal.DefaultPrecision := precisions;                 // ZZxA
  BigDecimal.DefaultRoundingMode := rmBack;           // Zۂ߃[hA
end;

// xk[C
// Akiyama-Tanigawa algorithm
// BigInteger
procedure Bernoulli_number_BigInteger;
const
  n = (NB + 1) * 2;
var
  m, j, k, dpcs: integer;
  a     : array of BigInteger;      // q
  b     : array of BigInteger;      // 
  tmpN  : BigInteger;               // q
  tmpD  : BigInteger;               // 
  gcd   : BigInteger;               // ő
  b0    : BigInteger;
  ad, bd : bigdecimal;
begin
  setlength(a, n + 1);
  setlength(b, n + 1);
  dpcs := BigDecimal.DefaultPrecision;
  k := 0;
  for m := 0 to n do begin
    a[m] := 1;                                      // q l
    b[m] := (m + 1);                                //  l
    for j := m - 1 downto 0 do begin
      tmpN := a[j] * b[j + 1] - a[j + 1] * b[j];    // q
      tmpN := tmpN * (j + 1);                       // V
      tmpD := b[j] * b[j + 1];                      // 
      gcd := gcd_BigInteger(tmpN, tmpD);            // ő
      a[j] := tmpN div gcd;
      b[j] := tmpD div gcd;
    end;
    ad := a[0];
    ad := ad.RoundToPrecision(dpcs);                // dpcsɊۂ
    if (m > 0) and (m mod 2 = 0) then begin
      b0 := b[0];                                   // log֐pɕlB̒lvZ
      b0 := b0 * m * (m -1);                        // m xk[CNo
      bd := b0;
      bd := bd.RoundToPrecision(dpcs);              // dpcsɊۂ
      BM[k] := ad / bd;
      inc(k);
    end;
  end;
end;

// z ͒l f
// eps l
// dpcs L
// ndis \ true@\ false \
function log_big_sub(z : Combig; eps : bigdecimal; dpcs: integer; ndis: boolean): Combig;
const
  NN = 1000000;
var
  s, x, zm1, zp1, xh2, xg, n2p1 : ComBig;
  one, two : Combig;
  xgsn : Combig;
  asabs : bigdecimal;
  n : integer;
begin
  one.r := Bigdecimal.One;      // 1
  one.i := Bigdecimal.Zero;     //
  two := cadd(one, one);        // 2
  zm1 := csub(z, one);          // z - 1
  zp1 := cadd(z, one);          // z + 1
  x := cdiv(zm1, zp1);          // x = (z-1)/(z+1)
  s.r := Bigdecimal.Zero;
  s.i := Bigdecimal.Zero;
  xh2 := cmul(x, x);            // x^2
  xg := x;                      // n = 0;    x^(2n+1)
  n2p1 := one;                  // 2n+1 = 1
  n := 0;
  repeat
    xg.r := xg.r.RoundToPrecision(dpcs);
    xg.i := xg.i.RoundToPrecision(dpcs);
    xgsn := cdiv(xg, n2p1);     // x^(2n+1) / (2n+1)
    s := cadd(s, xgsn);         // 
    xg := cmul(xg, xh2);        // x^(2n+1)
    n2p1 := cadd(n2p1, two);    // 2n + 1
    inc(n);
    asabs := cabs(xgsn);
  until  (asabs < eps) or (n > NN);
  s := cmul(s, two);
  result := s;
end;

// z   
// eps l
// dpcs L
// Ԃl ΐf
function log_big(z : combig; eps : bigdecimal; dpcs: integer): combig;
var
  zb : combig;
  z10n : combig;
  Zmax, ten, one, bn : bigdecimal;
  pi_big2: bigdecimal;
begin
  if (z.r = bigdecimal.Zero) and (z.i = bigdecimal.Zero) then begin
    result.r := bigdecimal.Zero;
    result.i := bigdecimal.Zero;
    exit;
  end;

  pi_big2 := paib / bigdecimal.Two;     // paib = pi_bigs

  zb := z;

  // ƎƂ[łȂȂ 傫ȒlɂĉZ덷܂
  if (zb.r <> bigdecimal.Zero) and (zb.i <> bigdecimal.Zero) then begin
    if zb.i.Abs(zb.i) > zb.r.Abs(zb.r) then begin     // ̐Βl̐Βl傫ւ
      z.r := zb.i;
      z.i := zb.r;
      if zb.i < bigdecimal.Zero then begin            // 畄]
        z.r := -z.r;
        z.i := -z.i;
      end;
    end
    else begin                                        // ̐Βl̐ΒlƓ
      if zb.r < bigdecimal.Zero then begin            // ̒l畄]
        z.r := -zb.r;
        z.i := -zb.i;
      end;
    end;
  end;

  // z.r0z.i[łȂ  Ǝւ
  if (zb.r = bigdecimal.Zero) and (zb.i <> bigdecimal.Zero) then begin
    z.r := zb.i;
    z.i := zb.r;
    if z.r < bigdecimal.Zero then z.r := -z.r;      // 畄] [Ȃ̂ł̂܂
  end;

  // ̂ŋ[̎𔽓] [Ȃ̂ł̂܂
  if (zb.r <> bigdecimal.Zero) and (zb.i = bigdecimal.Zero) then
    if zb.r < bigdecimal.Zero then z.r := -zb.r;

  // 10̑ΐ𗘗pđ傫ȒľvZ𑬂܂
  ten := '10';
  ten := ten.RoundToPrecision(dpcs);               // dpcs킹
  one := bigdecimal.One;
  bn := bigdecimal.Zero;
  if not Z10F then begin
    z10.r := ten;                                       // z10 : cbig Z!0F : boolean  main.pas
    z10.i := bigdecimal.Zero;
    z10 := log_big_sub(z10, eps, dpcs, false);          // 10̑ΐ
    Z10F := true;
  end;
  zmax := cabs(z);                                // z̐Βl
  zmax := zmax.RoundToPrecision(dpcs);             // dpcs킹
  z.r := z.r.RoundToPrecision(dpcs);               // dpcs킹
  z.i := z.i.RoundToPrecision(dpcs);               // dpcs킹
  // 10ŏZ͏ZĒl̒l1`10͈̔͂ɓ悤ɂ܂B
  repeat
    if zmax > ten then begin        // zmax10傫10ŏZ
      z.r := z.r / ten;
      z.i := z.i / ten;
      zmax := zmax / ten;
      bn := bn + one;               // 10ŏZ +ɂȂ܂B
    end;
    if zmax < one then begin        // zmax1菬10Z
      z.r := z.r * ten;
      z.i := z.i * ten;
      zmax := zmax * ten;
      bn := bn - one;               // 10Z -ɂȂ܂B
    end;
  until (zmax <= ten) and (zmax >= one);
  //----------------------------------------

  // CꂽlőΐvZ
  result := log_big_sub(z, eps, dpcs, true);

  // 10̑ΐ␳
  z10n.r := z10.r * bn;             // ZΐlvZ10̑ΐln{
  z10n.i := z10.i * bn;

  result.r := result.r + z10n.r;          // 10̑ΐln{Z
  result.i := result.i + z10n.i;
  //----------------------------------------

  // z.r[z.i[łȂ /2␳
  if (zb.r = bigdecimal.Zero) and (zb.i <> bigdecimal.Zero) then begin
    if zb.i >= bigdecimal.Zero then result.i := result.i + pi_big2
                               else result.i := result.i - pi_big2;
  end;

  // z.r̒l̂ƕύXČvZĂ̋Ε␳B
  if (zb.r < bigdecimal.Zero) and (zb.i = bigdecimal.Zero) then
    result.i := result.i + pi_big;

  // ƎƂ[łȂ
  if (zb.r <> bigdecimal.Zero) and (zb.i <> bigdecimal.Zero) then begin
    // Βl苕̐Βl傫ꍇ
    if zb.i.Abs(zb.i) > zb.r.Abs(zb.r) then begin
      if zb.i > bigdecimal.Zero then result.i := -result.i + pi_big2    // /2␳
                                else result.i := -result.i - pi_big2;
    end
    // ̎
    else begin
      if zb.r < bigdecimal.Zero then
        if zb.i > bigdecimal.Zero then result.i := result.i + pi_big    // Ε␳
                                  else result.i := result.i - pi_big
    end;
  end;
end;


// OK}{
procedure log_GammaMul(x: combig; var ans : combig);
var
  xx, v, w : combig;
  tmp, tmp0, s, cans : combig;
  i: integer;
begin
  xx := x;                            // x->xx  Rs[Ďgp
  v := cone;                          // 1 + 0i
  tmp.i := bigdecimal.Zero;           // 0 + 0i
  tmp.r := NB;                        // NB + 0i
  while xx.r < tmp.r do begin
    v := cmul(v, xx);                 // v = v * x
    // JԂvZɂ錅h~
    v := combiground(v);
    xx := cadd(xx, cone);             // x := x + 1
  end;
  tmp := cmul(xx, xx);                // x^2
  w := cdiv(cone, tmp);               // w = 1 / x^2
  s := combiground(s);
  tmp.i := bigdecimal.Zero;
  for i := NB downto 1 do begin
    tmp.r := s.r + BM[i];             // tmp = s + B[i]
    tmp.i := s.i;
    s := cmul(tmp, w);                // s = tmp * w
    // JԂvZɂ錅h~
    s := combiground(s);
  end;
  tmp.r := s.r + BM[0];               // tmp = s + B[0]
  tmp.i := s.i;
  s := cdiv(tmp, xx);                 // s = (s + B[0]) / x
  s := cadd(s, log_2pis2);            // s = s + ln(2)/2
  tmp := log_big(v, epsb, Precisions);      // ln(v)
  s := csub(s, tmp);                  // s := s - ln(v)
  s := csub(s, xx);                   // s := s - x
  tmp := cdiv(cone, ctwo);            // tmp = 1/2
  tmp0 := csub(xx, tmp);              // tmp0 = x - 1/2
  tmp := log_big(xx, epsb, Precisions);                   // ln(x)
  tmp0 := cmul(tmp0, tmp);            // tmp0 = (x - 1/2) * ln(x)
  cans := cadd(s, tmp0);              // ans = s + (x - 1/2) * ln(x)
  // 𓚌킹
  cans := combiground(cans);
  ans := cans;
end;

// sin(z)
// eps l
// dpcs L
// Ԃl sinf
function sin_big(z : Combig; eps : bigdecimal; dpcs: integer): Combig;
var
  zh2n1, zh2: combig;
  fa2n1, fn, one : bigdecimal;
  s, sb, tmp : combig;
  abss: bigdecimal;
  FG : boolean;
begin
  // n = 0
  one := bigdecimal.One;
  zh2n1 := z;                     // z^(2n+1)
  zh2 := cmul(z,z);               // z^2
  fa2n1 := one;                   // (2n+1)!
  fn := one;                      // (2*0+1)!
  s := z;                         // z
  FG := false;                    // ͊
  // n = 1 `
  repeat
    sb := s;
    zh2n1 := cmul(zh2n1, zh2);    // z^(2n+1) = z^(2n+1) * Z^2
    fn := fn + one;               // fn = fn + 1
    fa2n1 := fa2n1* fn;           // (2n+1)! = fa2n1*fn
    fn := fn + one;               // fn = fn + 1
    fa2n1 := fa2n1* fn;           // (2n+1)!  = fa2n1*fn
    // ZOɌ
    fa2n1 := fa2n1.RoundToPrecision(dpcs);
    zh2n1.r := zh2n1.r.RoundToPrecision(dpcs);
    zh2n1.i := zh2n1.i.RoundToPrecision(dpcs);
    tmp := cdivb(zh2n1, fa2n1);   // z^(2n+1) / (2n+1)!
    if FG then begin              // Ȃ
      s := cadd(s, tmp);          // Z
      FG := false;                // ͊
    end
    else begin                    // Ȃ
      s := csub(s, tmp);          // Z
      FG := true;                 // ͋
    end;
    abss := cabs(tmp);            // |tmp|
  until abss < eps;
  result := s;
end;

// exp(x)    eC[
// x ͒l f
// dpcs L
// eps l
// x / 8 ->  ans^8    8ŏZ vZ Ō8
function exp_big(x: Combig; dpcs: integer; eps : bigdecimal): Combig;
var
  xh, s, si: Combig;
  one, ass, eight, i, ih : bigdecimal;
begin
  one := bigdecimal.One;
  eight := bigdecimal.Ten - bigdecimal.Two;
  eight := eight.RoundToPrecision(dpcs);
  x.r := x.r.RoundToPrecision(dpcs);
  x.i := x.i.RoundToPrecision(dpcs);
  x := cdivb(x, eight);      // x/8
  i := one;
  ih := i;
  xh := x;
  s := x;
  repeat
    i := i + one;
    ih := ih * i;             // i!
    xh := cmul(xh, x);        // x^n
    xh.r := xh.r.RoundToPrecision(dpcs);
    xh.i := xh.i.RoundToPrecision(dpcs);
    si := cdivb(xh, ih);      // x^n/i!
    s := cadd(s, si);
    ass := cabs(si);
  until (ass < eps);
  s.r := s.r + one;
  s := cmul(s, s);            // x^2
  s := cmul(s, s);            // x^4
  result := cmul(s, s);       // x^8
  result.r := result.r.RoundToPrecision(dpcs);
  result.i := result.i.RoundToPrecision(dpcs);
end;

// {K}  f
// x̒l 0 ƕ̎́ɂȂ̂ŁtOZbgl1Ԃ܂B
function gammabig(xx : combig; var ans: combig) : integer;
var
  tmp, sinx, logG, cpai : combig;
  x : combig;
  czero, cans : combig;
  btwo, eps : bigdecimal;
begin
  czero.r := bigdecimal.Zero;
  czero.i := bigdecimal.Zero;
  btwo := bigdecimal.Two;
  eps := '1e-200';

  // xx̒lωxx̌lsɂȂ̂ŃRs[Ďgp܂
  x := xx;

  // [Ŏ[܂ݕ̐灇
  if (x.i = czero.i) and (x.r <= czero.r) then begin
    tmp.r := x.r.Frac;                    // ̏_ȉo
    if tmp.r = czero.r then begin         // _ȉ[
      ans.r := maxmpfbig;
      tmp.r := x.r / btwo;
      tmp.r := tmp.r.Frac;
      if tmp.r <> czero.r then ans.r := -ans.r; // 畄]
      ans.i := czero.i;
      result := 1;                        // tOZbg
      exit;                               // I
    end;
  end;
  cpai.r := paib;
  cpai.i := czero.i;                      // + 0i
  // Zp킹
  cpai := combiground(cpai);
  if x.r < czero.r then begin             // x.real < 0
    tmp := cmul(x, cpai);                 // x*
    tmp := combiground(tmp);
    sinx := sin_big(tmp, eps, Precisions);      // sin(x);
    tmp := csub(cone, x);                 // 1-x
    log_GammaMul(tmp, LogG);              // loggamma(1-x);
    logG := exp_big(logG, Precisions, eps);     // exp(loggamma)
    cpai := combiground(cpai);
    sinx := combiground(sinx);
    tmp := cdiv(cpai, sinx);              // /sin(x)
    logG := combiground(logG);
    tmp := combiground(tmp);
    cans := cdiv(tmp, logG);
  end
  else begin                              // x.real >= 0
    log_GammaMul(x, logG);                // logGamma(x)
    cans := exp_big(logG, Precisions, eps);     // exp(loggamma)
  end;
  ans := combiground(cans);
  result := 0;                            // tOl
end;

// x^y
// exp[ln(x) * y]
// A[ɂȂłAZ덷ɂA[ɂȂȂ̂āAċI
// Ƀ[ɂĂ܂B
function pow_big(x, y : combig; eps : bigdecimal): combig;
var
  dpcs : integer;
  harf, absi, absy, four : bigdecimal;
  lnx, tmp : combig;
begin
  if (x.r = bigdecimal.zero) and (x.i = bigdecimal.zero) then begin
    result.r := bigdecimal.Zero;
    result.i := bigdecimal.Zero;
    exit;
  end;
  dpcs := BigDecimal.DefaultPrecision;
  lnx := log_big(x, eps, dpcs);               // ln(x)
  lnx := combiground(lnx);                // ppcs ۂ
  y := combiground(y);                    // ppcs ۂ
  tmp := cmul(lnx, y);
  tmp := combiground(tmp);                // ln(x) * y
  result := exp_big(tmp, dpcs, eps);

  // x̋y̋0Ȃ
  if (x.i = bigdecimal.zero) and (y.i = bigdecimal.Zero) then begin
    harf := bigdecimal.One / bigdecimal.Two;          // 0.5
    absy := y.r.Abs(y.r);       // y̎̐Βl
    absi := absy.Frac;          // y̎̐Βl̏
    absi := absi - harf;        // y̎̐Βl̏0.5̍
    // [(y̎̏0.5)
    if absi = bigdecimal.Zero then
      // x̎0Ɠ0傫Ȃ
      if x.r >= bigdecimal.zero then result.i := bigdecimal.zero   // ̋0
                                // 0菬Ȃ
                                else result.r := bigdecimal.zero;  // ̎0
    absi := y.r.Frac;           // 搔̏
    if absi = bigdecimal.Zero then result.i := bigdecimal.Zero;    // Ȃ瓚̋0
  end;
  // x̐y̋0Ȃ
  if (x.r = bigdecimal.Zero) and (y.i = bigdecimal.Zero) then begin
    absi := y.r.Frac;                                     // 搔̏
    if absi = bigdecimal.Zero then begin                  // Ȃ
      absy := y.r / bigdecimal.Two;                       // mF
      absi := absy.Frac;                                  // 
      if absi = bigdecimal.Zero then result.i := bigdecimal.Zero             // Ȃ狕0
                                else result.r := bigdecimal.Zero;            // Ȃ0
    end;
  end;
  absy := x.r.Abs(x.r);                                     // |x.r|
  absi := x.i.Abs(x.i);                                     // |x.i|
  // x̎Ƌ̐Βly.r[łȂ
  if (absy = absi) and (y.i = bigdecimal.Zero) and not (y.r = bigdecimal.Zero) then begin
    absi := y.r.Frac;                                       // 搔̏
    if absi = bigdecimal.Zero then  begin                   // Ȃ
      absy := y.r / bigdecimal.Two;                         // mF
      absi := absy.Frac;                                    // 
      if absi = bigdecimal.Zero then begin                  // Ȃ
        four := bigdecimal.Two + bigdecimal.Two;            // 4
        absy := y.r / four;                                 // 4mF
        absi := absy.Frac;                                  // 
        if absi = bigdecimal.Zero then result.i := bigdecimal.Zero      // Ȃ 0  y= 4 8
                                  else result.r := bigdecimal.Zero;     // Ȃ 0  y= 2 6 10
      end;
    end;
  end;
end;

// lݒ
initialization
  precisions := DPS;
  BigDecimal.DefaultPrecision := precisions;
  zero := bigdecimal.Zero;
  one := bigdecimal.One;
  cone.r := bigdecimal.One;
  cone.i := bigdecimal.Zero;              // 1 + 0i
  ctwo.r := bigdecimal.Two;
  ctwo.i := bigdecimal.Zero;              // 2 + 0i

  paib := pi_big;                         // 
  maxmpfbig := '1.0e+1000000';            // L Zől K}vZől
  epsb := EPSC;
  tmpb.r := paib + paib;                  // 2
  tmpb.i := bigdecimal.Zero;;
  log_2pis2 := log_big(tmpb, epsb, DPS);  // Ln(2)
  // ZO킹
  log_2pis2.r := log_2pis2.r.RoundToPrecision(DPS);               // dpcsɊۂ
  log_2pis2.i := log_2pis2.i.RoundToPrecision(DPS);               // dpcsɊۂ

  log_2pis2 := cdiv(log_2pis2, ctwo);     // Ln(2)/2

  Bernoulli_number_BigInteger;            // xk[Cz쐬

end.
